# hangul_ends_with_consonant.py

from hangulpy.utils import is_hangul_char, HANGUL_BEGIN_UNICODE, CHOSUNG_LIST, JONGSUNG_LIST

def ends_with_consonant(char):
    """
    주어진 문자가 자음으로 끝나는지 확인합니다.
    
    :param char: 확인할 문자
    :return: 자음으로 끝나면 True, 아니면 False
    """
    if not is_hangul_char(char):
        return char in CHOSUNG_LIST

    # 한글 음절의 유니코드 값을 기준으로 종성 인덱스를 확인합니다.
    char_index = ord(char) - HANGUL_BEGIN_UNICODE
    jongsung_index = char_index % 28  # 28개의 종성

    return JONGSUNG_LIST[jongsung_index] != ''



# hangulpy/__init__.py

from .chosung import chosungIncludes, get_chosung_string, get_chosung
from .hangul_check import is_hangul_consonant, is_hangul_vowel
from .hangul_contains import hangul_contains
from .hangul_decompose import decompose_hangul_char
from .hangul_ends_with_consonant import ends_with_consonant
from .hangul_number import float_to_hangul, hangul_to_number, number_to_hangul
from .hangul_role import can_be_chosung, can_be_jongsung
from .hangul_split import split_hangul_char, split_hangul_string
from .josa import has_jongsung, josa


# hangul_number.py

from hangulpy.utils import UNITS, NUMBERS

def number_to_hangul(num):
    """
    주어진 숫자를 한글로 변환합니다.
    
    :param num: 변환할 숫자
    :return: 한글 숫자 문자열
    """
    if num == 0:
        return "영"
    
    if isinstance(num, float):
        return float_to_hangul(num)
    
    result = ""
    str_num = str(num)[::-1]
    
    for i, digit in enumerate(str_num):
        digit = int(digit)
        if digit != 0:
            if i == 4:
                result = "만" + result
            elif i > 4 and i % 4 == 0:
                result = UNITS[4 * (i // 4)] + result
            part = NUMBERS[digit] + UNITS[i % 4]
            if i % 4 == 0 and digit == 1 and i != 0:
                part = UNITS[i % 4]  # 1은 생략
            result = part + result

    if result.startswith('일'):
        result = result[1:]
    
    return result

def float_to_hangul(num):
    """
    주어진 소수를 한글로 변환합니다.
    
    :param num: 변환할 소수
    :return: 한글 숫자 문자열
    """
    int_part, frac_part = str(num).split('.')
    return f"{number_to_hangul(int(int_part))} 점 {''.join(NUMBERS[int(d)] for d in frac_part)}"

def hangul_to_number(hangul):
    """
    주어진 한글 숫자 문자열을 숫자로 변환합니다.
    
    :param hangul: 변환할 한글 숫자 문자열
    :return: 숫자
    """
    num_map = {name: value for value, name in enumerate(NUMBERS) if name}
    unit_map = {name: 10**idx for idx, name in enumerate(UNITS) if name}
    num = 0
    tmp = 0
    stack = []
    is_float = False
    float_part = 0
    float_div = 1

    # 소수점 분리
    if '점' in hangul:
        integer_part, fractional_part = hangul.split('점')
    else:
        integer_part, fractional_part = hangul, ''

    # 정수 부분 처리
    for char in integer_part:
        if char in num_map:
            stack.append(num_map[char])
        elif char in unit_map:
            if not stack:
                stack.append(1)
            tmp += sum(stack) * unit_map[char]
            stack = []
        else:
            if stack:
                tmp += sum(stack)
            stack = []
            num += tmp
            tmp = 0

    if stack:
        tmp += sum(stack)
    num += tmp

    # 소수 부분 처리
    if fractional_part:
        for char in fractional_part:
            if char in num_map:
                float_part = float_part * 10 + num_map[char]
                float_div *= 10
        return num + float_part / float_div

    return num



# josa.py

from hangulpy.utils import is_hangul_char, HANGUL_BEGIN_UNICODE, JONGSUNG_COUNT

def has_jongsung(char):
    """
    주어진 한글 음절에 받침이 있는지 확인합니다.
    
    :param char: 한글 음절 문자
    :return: 받침이 있으면 True, 없으면 False
    """
    if is_hangul_char(char):
        # 한글 음절의 유니코드 값을 기준으로 받침의 유무를 확인합니다.
        char_index = ord(char) - HANGUL_BEGIN_UNICODE
        return (char_index % JONGSUNG_COUNT) != 0
    return False

def josa(word, particle):
    """
    주어진 단어에 적절한 조사를 붙여 반환합니다.
    
    :param word: 조사와 결합할 단어
    :param particle: 붙일 조사 ('을/를', '이/가', '은/는', '와/과', '으로/로', '이나/나', '이에/에', '이란/란', '아/야', '이랑/랑', '이에요/예요', '으로서/로서', '으로써/로써', '으로부터/로부터', '이여/여', '께서', '이야/야', '와서/와', '이라서/라서', '이든/든', '이며/며', '이라도/라도', '이니까/니까', '이지만/지만', '이랑는/랑는')
    :return: 적절한 조사가 붙은 단어 문자열
    """
    if not word:
        return ''
    
    # 단어의 마지막 글자를 가져옵니다.
    word_ending = word[-1]
    # 마지막 글자의 받침 유무를 확인합니다.
    jongsung_exists = has_jongsung(word_ending)
    
    if particle == '을/를':
        return word + ('을' if jongsung_exists else '를')
    elif particle == '이/가':
        return word + ('이' if jongsung_exists else '가')
    elif particle == '은/는':
        return word + ('은' if jongsung_exists else '는')
    elif particle == '와/과':
        return word + ('과' if jongsung_exists else '와')
    elif particle == '으로/로':
        return word + ('으로' if jongsung_exists else '로')
    elif particle == '이나/나':
        return word + ('이나' if jongsung_exists else '나')
    elif particle == '이에/에':
        return word + ('이에' if jongsung_exists else '에')
    elif particle == '이란/란':
        return word + ('이란' if jongsung_exists else '란')
    elif particle == '아/야':
        return word + ('아' if jongsung_exists else '야')
    elif particle == '이랑/랑':
        return word + ('이랑' if jongsung_exists else '랑')
    elif particle == '이에요/예요':
        return word + ('이에요' if jongsung_exists else '예요')
    elif particle == '으로서/로서':
        return word + ('으로서' if jongsung_exists else '로서')
    elif particle == '으로써/로써':
        return word + ('으로써' if jongsung_exists else '로써')
    elif particle == '으로부터/로부터':
        return word + ('으로부터' if jongsung_exists else '로부터')
    elif particle == '이여/여':
        return word + ('이여' if jongsung_exists else '여')
    elif particle == '께서':
        return word + '께서'
    elif particle == '이야/야':
        return word + ('이야' if jongsung_exists else '야')
    elif particle == '와서/와':
        return word + ('와서' if jongsung_exists else '와')
    elif particle == '이라서/라서':
        return word + ('이라서' if jongsung_exists else '라서')
    elif particle == '이든/든':
        return word + ('이든' if jongsung_exists else '든')
    elif particle == '이며/며':
        return word + ('이며' if jongsung_exists else '며')
    elif particle == '이라도/라도':
        return word + ('이라도' if jongsung_exists else '라도')
    elif particle == '이니까/니까':
        return word + ('이니까' if jongsung_exists else '니까')
    elif particle == '이지만/지만':
        return word + ('이지만' if jongsung_exists else '지만')
    elif particle == '이랑는/랑는':
        return word + ('이랑는' if jongsung_exists else '랑는')
    else:
        raise ValueError(f"Unsupported particle: {particle}")
